/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.modules.curriculum.manager;

import java.util.Date;
import java.util.List;

import org.olat.core.commons.persistence.DB;
import org.olat.modules.curriculum.CurriculumElementType;
import org.olat.modules.curriculum.model.CurriculumElementTypeImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 
 * Initial date: 14 févr. 2018<br>
 * @author srosse, stephane.rosse@frentix.com, http://www.frentix.com
 *
 */
@Service
public class CurriculumElementTypeDAO {
	
	@Autowired
	private DB dbInstance;
	
	public CurriculumElementType createCurriculumElementType(String identifier, String displayName, String description, String externalId) {
		CurriculumElementTypeImpl type = new CurriculumElementTypeImpl();
		type.setCreationDate(new Date());
		type.setLastModified(type.getCreationDate());
		type.setIdentifier(identifier);
		type.setDisplayName(displayName);
		type.setDescription(description);
		type.setExternalId(externalId);
		dbInstance.getCurrentEntityManager().persist(type);
		return type;
	}
	
	public CurriculumElementType loadByKey(Long key) {
		List<CurriculumElementType> types = dbInstance.getCurrentEntityManager()
				.createNamedQuery("loadCurriculumElementTypeByKey", CurriculumElementType.class)
				.setParameter("key", key)
				.getResultList();
		return types == null || types.isEmpty() ? null : types.get(0);
	}
	
	public CurriculumElementType update(CurriculumElementType type) {
		((CurriculumElementTypeImpl)type).setLastModified(new Date());
		return dbInstance.getCurrentEntityManager().merge(type);
	}

}
