/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.user.ui.organisation;

import java.util.ArrayList;
import java.util.List;

import org.olat.basesecurity.OrganisationRoles;
import org.olat.core.gui.UserRequest;
import org.olat.core.gui.components.Component;
import org.olat.core.gui.components.link.Link;
import org.olat.core.gui.components.link.LinkFactory;
import org.olat.core.gui.components.velocity.VelocityContainer;
import org.olat.core.gui.control.Event;
import org.olat.core.gui.control.WindowControl;
import org.olat.core.gui.control.controller.BasicController;
import org.olat.user.ui.organisation.event.RoleEvent;

/**
 * 
 * Initial date: 29 mars 2018<br>
 * @author srosse, stephane.rosse@frentix.com, http://www.frentix.com
 *
 */
public class RoleListController extends BasicController {

	private final VelocityContainer mainVC;

	public RoleListController(UserRequest ureq, WindowControl wControl, OrganisationRoles[] roles) {
		super(ureq, wControl);
		
		mainVC = createVelocityContainer("tools");
		List<String> links = new ArrayList<>(roles.length);
		for(OrganisationRoles role:roles) {
			addLink(role, links);
		}
		mainVC.contextPut("links", links);
		putInitialPanel(mainVC);
	}
	
	private Link addLink(OrganisationRoles role, List<String> links) {
		Link link = LinkFactory.createLink(role.name(), role.name(), getTranslator(), mainVC, this, Link.LINK);
		link.setUserObject(role);
		mainVC.put(role.name(), link);
		links.add(role.name());
		return link;
	}

	@Override
	protected void doDispose() {
		//
	}

	@Override
	protected void event(UserRequest ureq, Component source, Event event) {
		if(source instanceof Link) {
			Link link = (Link)source;
			if(link.getUserObject() instanceof OrganisationRoles) {
				fireEvent(ureq, new RoleEvent((OrganisationRoles)link.getUserObject()));
			}
		}
	}
}
